@@ -139,7 +139,9 @@ use Text::PDF::Number;
 use Text::PDF::Objind;
 use Text::PDF::String;
 
-$VERSION = "0.04";      # MJPH  14-JUL-1999     Correct paths for tarball release
+$VERSION = "0.06";      # MJPH  11-SEP-1999   Sort out unixisms
+#$VERSION = "0.05";      # MJPH   9-SEP-1999     Add ship_out
+#$VERSION = "0.04";      # MJPH  14-JUL-1999     Correct paths for tarball release
 #$VERSION = "0.03";      # MJPH  14-JUL-1999     Correct paths for tarball release
 #$VERSION = "0.02";     # MJPH  30-JUN-1999     Transfer from old library
 
@@ -250,7 +252,7 @@ sub append_file
     { $tdict->{'Root'} = $self->{' newroot'}; }
     else
     { $tdict->{'Root'} = $self->{'Root'}; }
-    $tdict->{'Size'} = PDFNum($self->{'Size'}->val + $#{$self->{' outlist'}} + 1);
+    $tdict->{'Size'} = $self->{'Size'};
     $fh = $self->{' INFILE'};
     $fh->seek($self->{' epos'}, 0);
     $self->out_trailer($fh, $tdict);
@@ -270,8 +272,27 @@ $fname may be an IO object;
 sub out_file
 {
     my ($self, $fname) = @_;
-    my ($fh, $tdict);
 
+    $self->create_file($fname);
+    $self->close_file;
+    $self;
+}
+
+
+=head2 $p->create_file($fname)
+
+Creates a new output file (no check is made of an existing open file) of
+the given filename or IO object. Note, make sure that $p->{'Version'} is set
+correctly before calling this function.
+
+=cut
+
+sub create_file
+{
+    my ($self, $fname) = @_;
+    my ($fh);
+
+    $self->{' fname'} = $fname;
     if (ref $fname)
     { $fh = $fname; }
     else
@@ -280,22 +301,38 @@ sub out_file
         binmode $fh;
     }
 
+    $self->{' OUTFILE'} = $fh;
+    $fh->print("%PDF-1." . ($self->{'Version'} || "2") . "\n");
+    $fh->print("%쏢\n");              # and some binary stuff in a comment
+    $self;
+}
+
+
+=head2 $p->close_file
+
+Closes up the open file for output by outputting the trailer etc.
+
+=cut
+
+sub close_file
+{
+    my ($self) = @_;
+    my ($fh, $tdict);
+    
     $tdict = PDFDict();
     $tdict->{'Info'} = $self->{'Info'} if defined $self->{'Info'};
     $tdict->{'Root'} = $self->{' newroot'} ne "" ? $self->{' newroot'} : $self->{'Root'};
 
 # remove all freed objects from the outlist
     @{$self->{' outlist'}} = grep(!$_->{' isfree'}, @{$self->{' outlist'}});
-    $tdict->{'Size'} = PDFNum($#{$self->{' outlist'}} + 2);
-    $fh->print("%PDF-1." . ($self->{'Version'} || "3") . "\n");
-    $fh->print("%쏢\n");              # and some binary stuff in a comment
-    $self->out_trailer($fh, $tdict);
-    close($fh);
-    MacPerl::SetFileInfo("CARO", "TEXT", $fname) if ($^O eq "MacOS" && !ref($fname));
+    $tdict->{'Size'} = PDFNum(1);
+    $self->out_trailer($tdict);
+    close($self->{' OUTFILE'});
+    MacPerl::SetFileInfo("CARO", "TEXT", $self->{' fname'})
+            if ($^O eq "MacOS" && !ref($self-{' fname'}));
     $self;
 }
 
-
 =head2 ($value, $str) = $p->readval($str)
 
 Reads a PDF value from the current position in the file. If $str is too short
@@ -444,7 +481,7 @@ sub read_obj
     my ($self, $objind) = @_;
     my ($loc, $res, $str, $oldloc);
 
-    return ($objind) if $self->{' objects'}{$objind->uid};
+#    return ($objind) if $self->{' objects'}{$objind->uid};
     $loc = $self->locate_obj($objind->{' objnum'}, $objind->{' objgen'});
     $oldloc = $self->{' INFILE'}->tell;
     $self->{' INFILE'}->seek($loc, 0);
@@ -559,11 +596,52 @@ sub remove_obj
 # who says it has to be fast
     delete $self->{' objects'}{$objind->uid};
     @{$self->{' outlist'}} = grep($_ ne $objind, @{$self->{' outlist'}});
+    @{$self->{' printed'}} = grep($_ ne $objind, @{$self->{' printed'}});
     $self->{' objcache'}{$objind->{' objnum'}, $objind->{' objgen'}} = undef
             if ($self->{' objcache'}{$objind->{' objnum'}, $objind->{' objgen'}} eq $objind);
     $self;
 }
 
+
+=head2 $p->ship_out(@objects)
+
+Ships the given objects (or all objects for output if @objects is empty) to
+the currently open output file (assuming there is one). Freed objects are not
+shipped, and once an object is shipped it is switched such that this file
+becomes its source and it will not be shipped again unless out_obj is called
+again. Notice that a shipped out object can be re-output or even freed, but
+that it will not cause the data already output to be changed.
+
+=cut
+
+sub ship_out
+{
+    my ($self, @objs) = @_;
+    my ($n, @outlist, $fh, $objind);
+
+    return unless defined($fh = $self->{' OUTFILE'});
+    seek($fh, 0, 2);            # go to the end of the file
+
+    @objs = @{$self->{' outlist'}} unless ($#objs >= 0);
+    foreach $objind (@objs)
+    {
+        next unless $objind->is_obj($self);
+        @outlist = grep {$_ ne $objind} @{$self->{' outlist'}};
+        next if ($#outlist == $#{$self->{' outlist'}});
+        @{$self->{' outlist'}} = @outlist;
+        next if grep {$_ eq $objind} @{$self->{' free'}};
+
+        $self->{' locs'}{$objind->uid} = $fh->tell;
+        $fh->printf("%d %d obj\n", @{$self->{' objects'}{$objind->uid}}[0..1]);
+        $objind->outobjdeep($fh, $self);
+        $fh->print("\nendobj\n");
+        push (@{$self->{' printed'}}, $objind)
+                unless (grep {$_ eq $objind} @{$self->{' printed'}});
+    }
+    $self;
+}
+
+
 =head1 PRIVATE METHODS & FUNCTIONS
 
 The following methods and functions are considered private to this class. This
@@ -727,27 +805,31 @@ freed ones. It then outputs the trailing dictionary and the trailer code.
 
 sub out_trailer
 {
-    my ($self, $fh, $tdict) = @_;
+    my ($self, $tdict) = @_;
     my ($objind, $j, $i, $iend, @xreflist, $first, $k, $xref, $tloc, @freelist);
-    my (%locs);
-    my ($size) = $#{$self->{' outlist'}};
+    my (%locs, $size);
+    my ($fh) = $self->{' OUTFILE'};
+
+    while (@{$self->{' outlist'}})
+    { $self->ship_out; }
     
-    foreach $objind (@{$self->{' outlist'}})
-    {
-        next if ($self->{' objects'}{$objind->uid}[2]);
-        $locs{$objind->uid} = $fh->tell;
-        $fh->printf("%d %d obj\n", @{$self->{' objects'}{$objind->uid}}[0..1]);
-        $objind->outobjdeep($fh, $self);
-        $fh->print("\nendobj\n");
-    }
-    if ($#{$self->{' outlist'}} > $size)
-    { $tdict->{'Size'} = PDFNum($tdict->{'Size'}->val + $#{$self->{' outlist'}} - $size); }
+#    foreach $objind (@{$self->{' outlist'}})
+#    {
+#        next if ($self->{' objects'}{$objind->uid}[2]);
+#        $locs{$objind->uid} = $fh->tell;
+#        $fh->printf("%d %d obj\n", @{$self->{' objects'}{$objind->uid}}[0..1]);
+#        $objind->outobjdeep($fh, $self);
+#        $fh->print("\nendobj\n");
+#    }
+    $size = @{$self->{' printed'}} + @{$self->{' free'}};
+    $tdict->{'Size'} = PDFNum($tdict->{'Size'}->val + $size);
 
     $tloc = $fh->tell;
     $fh->print("xref\n");
 
     @xreflist = sort {$self->{' objects'}{$a->uid}[0] <=>
-                $self->{' objects'}{$b->uid}[0]} @{$self->{' outlist'}};
+                $self->{' objects'}{$b->uid}[0]}
+                        (@{$self->{' printed'}}, @{$self->{' free'}});
     @freelist = sort {$self->{' objects'}{$a->uid}[0] <=>
                 $self->{' objects'}{$b->uid}[0]} @{$self->{' free'}};
 
@@ -761,7 +843,7 @@ sub out_trailer
 #        }
         if ($i > $#xreflist || $self->{' objects'}{$xreflist[$i]->uid}[0] != $j + 1)
         {
-            $fh->print(($first == -1 ? "0 " : "$xreflist[$first]->{' objnum'} ") . ($i - $first) . "\n");
+            $fh->print(($first == -1 ? "0 " : "$self->{' objects'}{$xreflist[$first]->uid}[0] ") . ($i - $first) . "\n");
             if ($first == -1)
             {
                 $fh->printf("%010d 65535 f \n", $freelist[$k]);
@@ -779,7 +861,7 @@ sub out_trailer
                             " f \n"));
                 } else
                 {
-                    $fh->print(pack("A10AA5A4", sprintf("%010d", $locs{$xref->uid}), " ",
+                    $fh->print(pack("A10AA5A4", sprintf("%010d", $self->{' locs'}{$xref->uid}), " ",
                             sprintf("%05d", $self->{' objects'}{$xref->uid}[1]),
                             " n \n"));
                 }
